#include <QtGui>
#include "ViEngine.h"
#include "ViEditView.h"

ViEngine::ViEngine(QObject *parent)
	: m_noInsModeAtImeOpenStatus(false), m_mode(CMD), m_editor(0)
	, m_repeatCount(0), QObject(parent)
{
}

ViEngine::~ViEngine()
{

}
void ViEngine::setEditor(ViEditView *editor)
{
	m_editor = editor;
#if USE_EVENT_FILTER
	m_editor->installEventFilter(this);
#endif
}
void ViEngine::setMode(Mode mode)
{
	if( mode != m_mode ) {
		m_mode = mode;
		m_noInsModeAtImeOpenStatus = true;
		emit modeChanged(mode);
		m_noInsModeAtImeOpenStatus = false;
	}
}
#if 1
bool ViEngine::eventFilter(QObject *obj, QEvent *event)
{
	if( obj == m_editor && event->type() == QEvent::KeyPress ) {
		switch( mode() ) {
		case CMD:
			cmdModeKeyPressEvent(static_cast<QKeyEvent *>(event));	//	R}h[h̏ꍇ̏
			return true;
		case INSERT:
			return insModeKeyPressEvent(static_cast<QKeyEvent *>(event));	//	}[h̏ꍇ̏
		}
	}
	return false;
}
#else
bool ViEngine::processKeyPressEvent( QKeyEvent * event )
{
	switch( mode() ) {
	case CMD:
		return cmdModeKeyPressEvent(event);	//	R}h[h̏ꍇ̏
	case INSERT:
		return insModeKeyPressEvent(event);	//	}[h̏ꍇ̏
	}
	return false;
}
#endif
/**
	@return		R}hƂďꍇ true
*/
bool ViEngine::cmdModeKeyPressEvent(QKeyEvent *event)
{
	QString text = event->text();
	if( text.isEmpty() ) return false;
	bool rc = true;
	ushort ch = text[0].unicode();
	if( ch == '0' && m_repeatCount != 0 || ch >= '1' && ch <= '9' ) {
		m_repeatCount = m_repeatCount * 10 + (ch - '0');
		return true;
	}
	switch( ch ) {
	case ':':
		setMode(CMDLINE);	//	: ꂽex[h
		break;
	case 'i':
		setMode(INSERT);	//	i ꂽ}[h
		break;
	case 'h':
		m_editor->moveCursor(QTextCursor::Left, repeatCount());
		break;
	case ' ':
	case 'l':
		m_editor->moveCursor(QTextCursor::Right, repeatCount());
		break;
	case 'k':
		m_editor->moveCursor(QTextCursor::Up, repeatCount());
		break;
	case 'j':
		m_editor->moveCursor(QTextCursor::Down, repeatCount());
		break;
	default:
		rc = false;
		break;
	}
	m_repeatCount = 0;
	return rc;
}
bool ViEngine::insModeKeyPressEvent(QKeyEvent *event)
{
	if( event->key() == Qt::Key_Escape ) {
		setMode(CMD);		//	Esc ꂽR}h[h
		return true;
	}
	return false;
}
void ViEngine::processExCommand(const QString &text)
{
	setMode(CMD);
}
void ViEngine::onImeOpenStatusChanged()
{
	if( mode() == CMD && !m_noInsModeAtImeOpenStatus )
		setMode(INSERT);
}
